/*
 * squashfs with lzma compression
 *
 * Copyright (C) 2009, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: sqlzma.c,v 1.3 2009-03-10 08:42:02 Exp $
 */
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/vmalloc.h>

#include "LzmaDec.h"
#include "LzmaDec.c"

static void *SzAlloc(void *p, size_t size) { p = p; return vmalloc(size); }
static void SzFree(void *p, void *address) { p = p; vfree(address); }
static ISzAlloc g_Alloc = { SzAlloc, SzFree };

int LzmaUncompress(void *dst, int dstlen, void *src, int srclen)
{
	int res;
	SizeT inSizePure;
	ELzmaStatus status;
	SizeT outSize;

	if (srclen < LZMA_PROPS_SIZE)
	{
		memcpy(dst, src, srclen);
		return srclen;
	}
	inSizePure = srclen - LZMA_PROPS_SIZE;
	outSize = dstlen;
	res = LzmaDecode(dst, &outSize, src + LZMA_PROPS_SIZE, &inSizePure,
	                 src, LZMA_PROPS_SIZE, LZMA_FINISH_ANY, &status, &g_Alloc);
	srclen = inSizePure ;

	if ((res == SZ_OK) ||
		((res == SZ_ERROR_INPUT_EOF) && (srclen == inSizePure)))
		res = 0;
	return outSize;
}
